# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['poetry_publish', 'poetry_publish.tests', 'poetry_publish.utils']

package_data = \
{'': ['*']}

install_requires = \
['twine']

entry_points = \
{'console_scripts': ['publish = poetry_publish.self:publish_poetry_publish',
                     'update_rst_readme = '
                     'poetry_publish.self:update_poetry_publish_readme']}

setup_kwargs = {
    'name': 'poetry-publish',
    'version': '0.5.0',
    'description': 'Helper to build and upload a project that used poetry to PyPi, with prechecks',
    'long_description': '====================\nAbout poetry-publish\n====================\n\nHelper to build and upload a project that used poetry to PyPi, with prechecks:\n\n* User must confirm:\n\n    * If ``__version__`` contains \'dev\' or \'rc\'\n\n    * If git repository is not on ``main`` or ``master``\n\n* Abort publish if git repository contains changes\n\n* Abort if git repository is not up-to-date\n\n* Abort if ``poetry check`` fails\n\n* Abort if ``twine check`` fails\n\n* Abort if git version tag already exists\n\nAfter a successful upload to PyPi:\n\n* create a git version tag\n\n* git push tag to remote server\n\nCompatible Python Versions (see `tox.ini <https://github.com/jedie/poetry-publish/blob/master/tox.ini>`_ or `.travis.yml <https://github.com/jedie/poetry-publish/blob/master/.travis.yml>`_):\n\n* 3.10, 3.9, 3.8, 3.7, 3.6\n\n* PyPy3\n\n+---------------------------------+----------------------------------------------------+\n| |Build Status on github|        | `github.com/jedie/poetry-publish/actions`_         |\n+---------------------------------+----------------------------------------------------+\n| |Build Status on travis-ci.org| | `travis-ci.org/jedie/poetry-publish`_              |\n+---------------------------------+----------------------------------------------------+\n| |Coverage Status on codecov.io| | `codecov.io/gh/jedie/poetry-publish`_              |\n+---------------------------------+----------------------------------------------------+\n| |Status on landscape.io|        | `landscape.io/github/jedie/poetry-publish/master`_ |\n+---------------------------------+----------------------------------------------------+\n| |PyPi version|                  | `pypi.org/project/poetry-publish/`_                |\n+---------------------------------+----------------------------------------------------+\n\n.. |Build Status on github| image:: https://github.com/jedie/poetry-publish/workflows/test/badge.svg?branch=master\n.. _github.com/jedie/poetry-publish/actions: https://github.com/jedie/poetry-publish/actions?query=workflow%3Atest\n.. |Build Status on travis-ci.org| image:: https://travis-ci.org/jedie/poetry-publish.svg\n.. _travis-ci.org/jedie/poetry-publish: https://travis-ci.org/jedie/poetry-publish/\n.. |Coverage Status on codecov.io| image:: https://codecov.io/gh/jedie/poetry-publish/branch/master/graph/badge.svg\n.. _codecov.io/gh/jedie/poetry-publish: https://codecov.io/gh/jedie/poetry-publish\n.. |Status on landscape.io| image:: https://landscape.io/github/jedie/poetry-publish/master/landscape.svg\n.. _landscape.io/github/jedie/poetry-publish/master: https://landscape.io/github/jedie/poetry-publish/master\n.. |PyPi version| image:: https://badge.fury.io/py/poetry-publish.svg\n.. _pypi.org/project/poetry-publish/: https://pypi.org/project/poetry-publish/\n\n-------\nexample\n-------\n\n::\n\n    ~/repos/python-creole$ poetry run publish\n\n    Check if we are on "master" branch:\n    \tCall: \'git branch --no-color\'\n    OK\n\n    Set version in "pyproject.toml" to: v1.4.3\n    \tCall: \'poetry version 1.4.3\'\n\n    Bumping version from 1.4.3 to 1.4.3\n\n    check if if git repro is clean:\n    \tCall: \'git status --porcelain\'\n    OK\n\n    Run "poetry check":\n    OK\n\n    check if pull is needed\n    \tCall: \'git fetch --all\'\n\n    Fordere an von origin\n    \tCall: \'git log HEAD..origin/master --oneline\'\n    OK\n    \tCall: \'git push\'\n\n    Everything up-to-date\n\n    Cleanup old builds:\n    \tremove tree: /home/jens/repos/python-creole/dist\n\n    build but do not upload...\n    \tCall: \'poetry build\'\n    Build log file is here: \'publish.log\'\n\n    check git tag\n    OK\n\n    Upload to PyPi via poetry:\n    \tCall: \'poetry publish\'\n\n\n    Publishing python-creole (1.4.3) to PyPI\n     - Uploading python-creole-1.4.3.tar.gz 100%\n     - Uploading python_creole-1.4.3-py3-none-any.whl 100%\n\n    git tag version\n    \tCall: \'git tag v1.4.3\'\n\n\n    git push tag to server\n    \tCall: \'git push --tags\'\n\n    Total 0 (delta 0), reused 0 (delta 0)\n    To github.com:jedie/python-creole.git\n     * [new tag]         v1.4.3 -> v1.4.3\n\n-----------------------\nsetup usage in projects\n-----------------------\n\nCreate a publish hook in you project, e.g. create ``your_project/publish.py`` with:\n\n::\n\n    from pathlib import Path\n\n    import your_project\n    from poetry_publish.publish import poetry_publish\n\n\n    def publish():\n        poetry_publish(\n            package_root=Path(your_project.__file__).parent.parent,\n            version=your_project.__version__,\n        )\n\nAdd this to your poetry ``pyproject.toml``, e.g.:\n\n::\n\n    [tool.poetry.scripts]\n    publish = \'your_project:publish\'\n\nTo publish do this:\n\n::\n\n    ~$ cd your_project\n    ~/your_project$ poetry run publish\n\n**Note:** Don\'t miss the ``run`` ! It\'s not the same as ``poetry publish``\n\nbased on:\n`https://github.com/jedie/python-code-snippets/blob/master/CodeSnippets/setup_publish.py <https://github.com/jedie/python-code-snippets/blob/master/CodeSnippets/setup_publish.py>`_\n\n---------\nunittests\n---------\n\n::\n\n    # clone repository (or use your fork):\n    ~$ git clone https://github.com/jedie/poetry-publish.git\n    ~$ cd poetry-publish\n\n    # install or update poetry:\n    ~/poetry-publish$ make install-poetry\n\n    # install poetry-publish via poetry:\n    ~/poetry-publish$ make install\n\n    # Run pytest:\n    ~/poetry-publish$ make pytest\n\n    # Run pytest via tox with all environments:\n    ~/poetry-publish$ make tox\n\n    # Run pytest via tox with one Python version:\n    ~/poetry-publish$ make tox-py38\n    ~/poetry-publish$ make tox-py37\n    ~/poetry-publish$ make tox-py36\n\n------------\nmake targets\n------------\n\nTo see all make targets, just call ``make``:\n\n::\n\n    ~/poetry-publish$ make\n    help                 List all commands\n    install-poetry       install or update poetry\n    install              install python-poetry_publish via poetry\n    update               Update the dependencies as according to the pyproject.toml file\n    lint                 Run code formatters and linter\n    fix-code-style       Fix code formatting\n    tox-listenvs         List all tox test environments\n    tox                  Run pytest via tox with all environments\n    tox-py36             Run pytest via tox with *python v3.6*\n    tox-py37             Run pytest via tox with *python v3.7*\n    tox-py38             Run pytest via tox with *python v3.8*\n    tox-py39             Run pytest via tox with *python v3.9*\n    pytest               Run pytest\n    update-rst-readme    update README.rst from README.creole\n    publish              Release new version to PyPi\n\n=======\nhistory\n=======\n\n* *dev* - `compare v0.5.0...master <https://github.com/jedie/poetry-publish/compare/v0.5.0...master>`_\n\n    * TBC\n\n* v0.5.0 - 2022-07-19 - `compare v0.4.1...v0.5.0 <https://github.com/jedie/poetry-publish/compare/v0.4.1...v0.5.0>`_\n\n    * Test with Python 3.10\n\n    * Fix twine check call.\n\n    * Fix ``poetry publish`` call.\n\n* v0.4.1 - 2021-03-19 - `compare v0.4.0...v0.4.1 <https://github.com/jedie/poetry-publish/compare/v0.4.0...v0.4.1>`_\n\n    * Bugfix if git ``main`` branch is used, instead of ``master``\n\n* v0.4.0 - 2020-10-17 - `compare v0.3.2...v0.4.0 <https://github.com/jedie/poetry-publish/compare/v0.3.2...v0.4.0>`_\n\n    * Call ``twine check dist/*.*``, too.\n\n    * Some meta updates to project setup\n\n* v0.3.2 - 2020-10-16 - `compare v0.3.1...v0.3.2 <https://github.com/jedie/poetry-publish/compare/v0.3.1...v0.3.2>`_\n\n    * Create git annotated tags instead of a lightweight tag. `Contributed by sebhmg in #9 <https://github.com/jedie/poetry-publish/issues/9>`_\n\n    * Make ``python-creole`` to an optional dependency. Based on a contribution by sebhmg\n\n    * Allow ``main`` beside ``master`` as stable branch.\n\n    * Add ``make update``\n\n    * Update ``pytest.ini``\n\n    * Change supported Python version in ``pyproject.toml``\n\n    * Run tests with Python 3.9, too\n\n* v0.3.1 - 2020-02-19 - `compare v0.3.0...v0.3.1 <https://github.com/jedie/poetry-publish/compare/v0.3.0...v0.3.1>`_\n\n    * less restricted dependency specification\n\n    * Add: |poetry_publish.tests.test_project_setup.test_assert_rst_readme|}\n\n* v0.3.0 - 2020-02-10 - `compare v0.2.3...v0.3.0 <https://github.com/jedie/poetry-publish/compare/v0.2.3...v0.3.0>`_\n\n    * Poetry publish error -> fallback and use twine\n\n    * call ``poetry publish`` with ``-vvv``\n\n    * call ``git push`` with ``origin <current-branch>``\n\n    * add isort config\n\n* v0.2.3 - 2020-02-02 - `compare v0.2.2...v0.2.3 <https://github.com/jedie/poetry-publish/compare/v0.2.2...v0.2.3>`_\n\n    * make ``poetry_publish.tests.test_project_setup`` usable for external packages\n\n* v0.2.2 - 2020-02-01 - `compare v0.2.1...v0.2.2 <https://github.com/jedie/poetry-publish/compare/v0.2.1...v0.2.2>`_\n\n    * Fix missing project description on PyPi\n\n* v0.2.1 - 2020-02-01 - `compare v0.2.0...v0.2.1 <https://github.com/jedie/poetry-publish/compare/v0.2.0...v0.2.1>`_\n\n    * call "poetry version" after "branch is master" check\n\n    * add many tests\n\n    * test with PyPy v3, too\n\n    * Upload coverage reports\n\n    * fix code style\n\n    * update README\n\n* v0.2.0 - 2020-02-01 - `compare 92e584...v0.2.0 <https://github.com/jedie/poetry-publish/compare/92e584ed8532c577feb971a5d8630cc1929ad6eb...v0.2.0>`_\n\n    * first released version cut out from `python-creole <https://github.com/jedie/python-creole>`_\n\n.. |poetry_publish.tests.test_project_setup.test_assert_rst_readme| image:: poetry_publish.tests.test_project_setup.test_assert_rst_readme\n\nfirst source code was written 27.11.2008: `Forum thread (de) <http://www.python-forum.de/viewtopic.php?f=3&t=16742>`_\n\n-------------\nProject links\n-------------\n\n+--------+---------------------------------------------+\n| GitHub | `https://github.com/jedie/poetry-publish/`_ |\n+--------+---------------------------------------------+\n| PyPi   | `https://pypi.org/project/poetry-publish/`_ |\n+--------+---------------------------------------------+\n\n.. _https://github.com/jedie/poetry-publish/: https://github.com/jedie/poetry-publish/\n.. _https://pypi.org/project/poetry-publish/: https://pypi.org/project/poetry-publish/\n\n--------\ndonation\n--------\n\n* `paypal.me/JensDiemer <https://www.paypal.me/JensDiemer>`_\n\n* `Flattr This! <https://flattr.com/submit/auto?uid=jedie&url=https%3A%2F%2Fgithub.com%2Fjedie%2Fpoetry-publish%2F>`_\n\n* Send `Bitcoins <http://www.bitcoin.org/>`_ to `1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F <https://blockexplorer.com/address/1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F>`_\n\n------------\n\n``Note: this file is generated from README.creole 2022-07-19 12:48:48 with "python-creole"``\n',
    'author': 'JensDiemer',
    'author_email': 'git@jensdiemer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)
